// test_quicksort.cpp
// VERSION 2
// Glenn G. Chappell
// 12 Dec 2013
//
// For CS 202 Fall 2013
// Test program for function template quicksort
//
// History:
// - v2: Fixed signed/unsigned problem

#include "quicksort.h"  // For function template quicksort
#include "quicksort.h"  // Double-inclusion test

#include <iostream>
using std::cout;
using std::endl;
using std::cin;
#include <cstdlib>
using std::size_t;
#include <deque>
using std::deque;
#include <string>
using std::string;
#include <algorithm>
using std::equal;
using std::sort;
using std::reverse;


bool allGood = true;  // true if all tests pass so far


// test_quicksort_int_small
// Test for quicksort: int values, small datasets
// Prints results of tests, sets global allGood to false if any fail
void test_quicksort_int_small()
{
    cout << endl;
    cout << "Test suite: int values, small datasets" << endl;

    // Make deque
    const size_t DATASIZE = 10;
    int data[DATASIZE] = { 2, 2, 1, 5, 8, 7, 6, 2, 4, 3 };
    bool result;  // For pass/fail results

    {
    deque<int> d(data, data+DATASIZE);
    quicksort(d.rbegin()+5, d.rbegin()+5);
    int fdata[DATASIZE] = { 3, 4, 2, 6, 7, 8, 5, 1, 2, 2 };
    result = equal(d.rbegin(), d.rend(), fdata);
    cout << "  Empty range: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }

    {
    deque<int> d(data, data+DATASIZE);
    quicksort(d.rbegin()+5, d.rbegin()+6);
    int fdata[DATASIZE] = { 3, 4, 2, 6, 7, 8, 5, 1, 2, 2 };
    result = equal(d.rbegin(), d.rend(), fdata);
    cout << "  Singleton range: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }

    {
    deque<int> d(data, data+DATASIZE);
    quicksort(d.rbegin()+5, d.rbegin()+7);
    int fdata[DATASIZE] = { 3, 4, 2, 6, 7, 5, 8, 1, 2, 2 };
    result = equal(d.rbegin(), d.rend(), fdata);
    cout << "  Size-2 range #1: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }

    {
    deque<int> d(data, data+DATASIZE);
    quicksort(d.rbegin()+4, d.rbegin()+6);
    int fdata[DATASIZE] = { 3, 4, 2, 6, 7, 8, 5, 1, 2, 2 };
    result = equal(d.rbegin(), d.rend(), fdata);
    cout << "  Size-2 range #2: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }

    {
    deque<int> d(data, data+DATASIZE);
    quicksort(d.rbegin()+5, d.rbegin()+8);
    int fdata[DATASIZE] = { 3, 4, 2, 6, 7, 1, 5, 8, 2, 2 };
    result = equal(d.rbegin(), d.rend(), fdata);
    cout << "  Size-3 range: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }

    {
    deque<int> d(data, data+DATASIZE);
    quicksort(d.rbegin()+1, d.rbegin()+9);
    int fdata[DATASIZE] = { 3, 1, 2, 2, 4, 5, 6, 7, 8, 2 };
    result = equal(d.rbegin(), d.rend(), fdata);
    cout << "  Size-8 range: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }
}


// test_quicksort_int_large
// Test for quicksort: int values, large datasets
// Prints results of tests, sets global allGood to false if any fail
void test_quicksort_int_large()
{
    cout << endl;
    cout << "Test suite: int values, large datasets" << endl;

    bool result;  // For pass/fail results

    {
    deque<int> d;
    const size_t DATASIZE = 10000;
    for (size_t i = 0; i < DATASIZE; ++i)
    {
        d.push_back(DATASIZE+i);
        d.push_back(DATASIZE-i);
    }
    deque<int> d2 = d;
    sort(d2.begin(), d2.end());
    quicksort(d.rbegin(), d.rend());
    result = equal(d.rbegin(), d.rend(), d2.begin());
    cout << "  Large range #1: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }

    {
    deque<int> d;
    const size_t DATASIZE = 10000;
    for (size_t i = 0; i < DATASIZE; ++i)
    {
        d.push_back(DATASIZE+i);
        d.push_back(DATASIZE-i);
    }
    deque<int> d2 = d;
    reverse(d2.begin(), d2.end());
    sort(d2.begin()+1, d2.end()-1);
    quicksort(d.rbegin()+1, d.rend()-1);
    result = equal(d.rbegin(), d.rend(), d2.begin());
    cout << "  Large range #2: ";
    if (result)
    {
        cout << "Good" << endl;
    }
    else
    {
        cout << "FAIL!!! *******************************"
             << endl;
        allGood = false;
    }
    }
}


// main program
// Runs tests for quicksort, prints results
int main()
{
    cout << "Test function template quicksort" << endl;
    test_quicksort_int_small();
    test_quicksort_int_large();
    
    cout << endl;
    if (allGood)
        cout << "All tests pass!" << endl;
    else
        cout << "***** SOME TESTS FAILED *****" << endl;

    cout << endl;
    cout << "Press ENTER to quit ";
    while (cin.get() != '\n') ;

    return 0;
}

